# Copyright (C) 2022 Hottinger Bruel and Kjaer Benelux B.V.
# Schutweg 15a
# 5145 NP Waalwijk
# The Netherlands
# http://www.hbm.com

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Recorder module interface."""

from .connection import ConnectionHandler
from .ghsapi_states import (
    RETURN_KEY,
    GHSDigitalOutMode,
    GHSDigitalOutput,
    GHSEnableDisable,
    GHSReturnValue,
    from_string,
    to_string,
)


def get_channel_count(
    con_handle: ConnectionHandler, slot_id: str
) -> tuple[str, int | None]:
    """Retrieve the number of channels for a recorder.

    This method can be called by multiple connected clients at same
    time.

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).

    Returns:
       Tuple with status and number of channels for the recorder.
    """

    if not slot_id:
        return "NullPtrArgument", None

    channel_count_dict = {
        "SlotId": slot_id,
    }

    response_json = con_handle.send_request_wait_response(
        "GetChannelCount", channel_count_dict
    )

    if ("ChannelCount" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None

    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["ChannelCount"],
    )


def get_digital_output(
    con_handle: ConnectionHandler, slot_id: str, digital_output: str | int
) -> tuple[str, str | None]:
    """Retrieve the Digital Output Mode for a specified Output ID in a
    recorder.

    This method can be called by multiple connected clients at same
    time.

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).
        digital_output: The output number desired.

    Returns:
       Tuple with status and digital output mode for that output.
    """

    if not slot_id or not digital_output:
        return "NullPtrArgument", None

    if isinstance(digital_output, str) and digital_output in GHSDigitalOutput:
        digital_output_dict = {
            "SlotId": slot_id,
            "Output": from_string(digital_output, GHSDigitalOutput),
        }

    elif (
        isinstance(digital_output, int)
        and digital_output in GHSDigitalOutput.values()
    ):
        digital_output_dict = {"SlotId": slot_id, "Output": digital_output}

    else:
        return "InvalidOutputNumber", None

    response_json = con_handle.send_request_wait_response(
        "GetDigitalOutput", digital_output_dict
    )

    if ("DigitalOutMode" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None

    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        to_string(response_json["DigitalOutMode"], GHSDigitalOutMode),
    )


def get_recorder_enabled(
    con_handle: ConnectionHandler, slot_id: str
) -> tuple[str, str | None]:
    """Determine if recorder is enabled or disabled.

    This method can be called by multiple connected clients at same
    time.

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).

    Returns:
       Tuple with status and recorder enabled status.
    """

    if not slot_id:
        return "NullPtrArgument", None

    recorder_enabled_dict = {
        "SlotId": slot_id,
    }

    response_json = con_handle.send_request_wait_response(
        "GetRecorderEnabled", recorder_enabled_dict
    )

    if ("IsRecorderEnabled" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None

    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        to_string(response_json["IsRecorderEnabled"], GHSEnableDisable),
    )


def get_recorder_info(
    con_handle: ConnectionHandler, slot_id: str
) -> tuple[str, str | None, str | None, str | None, str | None]:
    """Determine type, name, serial number and firmware version
    information for a recorder.

    The recorderType, recorderName, serialNumber and firmwareVersion
    parameters are UTF-8 encoded.

    This method can be called by multiple connected clients at same
    time.

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).

    Returns:
       Tuple with status and recorder information.
    """

    if not slot_id:
        return "NullPtrArgument", None, None, None, None

    recorder_info_dict = {
        "SlotId": slot_id,
    }

    response_json = con_handle.send_request_wait_response(
        "GetRecorderInformation", recorder_info_dict
    )

    if (
        not any(
            key in response_json
            for key in [
                "RecorderType",
                "RecorderName",
                "SerialNumber",
                "FirmwareVersion",
            ]
        )
    ) or (response_json[RETURN_KEY] != GHSReturnValue["OK"]):
        return (
            to_string(response_json[RETURN_KEY], GHSReturnValue),
            None,
            None,
            None,
            None,
        )

    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["RecorderType"],
        response_json["RecorderName"],
        response_json["SerialNumber"],
        response_json["FirmwareVersion"],
    )


def get_sample_rate(
    con_handle: ConnectionHandler, slot_id: str
) -> tuple[str, float | None]:
    """Determine the sample rate for a recorder.

    This method can be called by multiple connected clients at same
    time.

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).

    Returns:
       Tuple with status and sample rate for a recorder.
    """

    if not slot_id:
        return "NullPtrArgument", None

    sample_rate_dict = {
        "SlotId": slot_id,
    }

    response_json = con_handle.send_request_wait_response(
        "GetSampleRate", sample_rate_dict
    )

    if ("SampleRate" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None

    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["SampleRate"],
    )


def set_digital_output(
    con_handle: ConnectionHandler,
    slot_id: str,
    digital_output: str | int,
    digital_output_mode: str | int,
) -> str:
    """Set the Digital Output Mode for a specified Output ID in a
    recorder.

    This method will only process requests from the connected client
    with the most privileges order (Privileges order: 1- Perception,
    2- GenDaq, 3- Other)

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).
        digital_output: The output number desired.
        digital_output_mode: The digital output mode to set on that
        output.

    Returns:
       String value representing request status.
    """

    if not slot_id or not digital_output or not digital_output_mode:
        return "NullPtrArgument"

    if isinstance(digital_output, str) and digital_output in GHSDigitalOutput:
        digital_output = from_string(digital_output, GHSDigitalOutput)

    elif (
        isinstance(digital_output, int)
        and digital_output in GHSDigitalOutput.values()
    ):
        pass

    else:
        return "InvalidOutputNumber"

    if (
        isinstance(digital_output_mode, str)
        and digital_output_mode in GHSDigitalOutMode
    ):
        digital_output_mode = from_string(
            digital_output_mode, GHSDigitalOutMode
        )

    elif (
        isinstance(digital_output_mode, int)
        and digital_output_mode in GHSDigitalOutMode.values()
    ):
        pass

    else:
        return "IncompatibleDigitalOutputMode"

    digital_output_dict = {
        "SlotId": slot_id,
        "DigitalOutMode": digital_output_mode,
        "Output": digital_output,
    }

    response_json = con_handle.send_request_wait_response(
        "SetDigitalOutput", digital_output_dict
    )

    return to_string(response_json[RETURN_KEY], GHSReturnValue)


def set_recorder_enabled(
    con_handle: ConnectionHandler,
    slot_id: str,
    enabled: str | int,
) -> str:
    """Enable or disable a recorder.

    The system needs to be idle before calling this function.

    This method will only process requests from the connected client
    with the most privileges order (Privileges order: 1- Perception,
    2- GenDaq, 3- Other)

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).
        enabled: Set to GHS_Enable/GHS_Disable to enable/disable the
        recorder.

    Returns:
       String value representing request status.
    """

    if not slot_id or not enabled:
        return "NullPtrArgument"

    if isinstance(enabled, str) and enabled in GHSEnableDisable:
        enabled = from_string(enabled, GHSEnableDisable)

    elif isinstance(enabled, int) and enabled in GHSEnableDisable.values():
        pass

    else:
        return "InvalidDataType"

    recorder_enabled_dict = {
        "SlotId": slot_id,
        "EnabledStatus": enabled,
    }

    response_json = con_handle.send_request_wait_response(
        "SetRecorderEnabled", recorder_enabled_dict
    )

    return to_string(response_json[RETURN_KEY], GHSReturnValue)


def set_sample_rate(
    con_handle: ConnectionHandler,
    slot_id: str,
    sample_rate: float,
) -> str:
    """Set the sample rate for a recorder.

    The system needs to be idle before calling this function.
    This function overwrites any previously set sample rate setting for
    the specified recorder.
    If the specified sample rate is not supported by the recorder, the
    sample rate is rounded to the nearest supported sample rate.

    This method will only process requests from the connected client
    with the most privileges order (Privileges order: 1- Perception,
    2- GenDaq, 3- Other)

    Args:
        con_handle: A unique identifier per mainframe connection.
        slot_id: The slot containing the recorder to get number of
        channels for (e.g. 'A' for the first slot).
        sample_rate: In samples per second.

    Returns:
       String value representing request status.
    """

    if not slot_id or not sample_rate:
        return "NullPtrArgument"

    if not isinstance(sample_rate, float):
        return "InvalidDataType"

    sample_rate_dict = {
        "SlotId": slot_id,
        "SampleRate": sample_rate,
    }

    response_json = con_handle.send_request_wait_response(
        "SetSampleRate", sample_rate_dict
    )

    return to_string(response_json[RETURN_KEY], GHSReturnValue)



# Copyright (C) 2022 Hottinger Bruel and Kjaer Benelux B.V.
# Schutweg 15a
# 5145 NP Waalwijk
# The Netherlands
# http://www.hbm.com

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Mainframe module interface."""

from .connection import ConnectionHandler
from .ghsapi_states import (
    RETURN_KEY,
    GHSReturnValue,
    GHSSyncStatus,
    GHSUserMode,
    from_string,
    to_string,
)


def get_mainframe_info(
    con_handle: ConnectionHandler,
) -> tuple[str, str | None, str | None, str | None, str | None]:
    """Interface to determine type, name, serial number and firmware
    version information for the connected mainframe.

    The mainframeType, mainframeName, serialNumber and firmwareVersion
    parameters are UTF-8 encoded.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       Tuple with type, name, serial number and firmware version.
    """

    response_json = con_handle.send_request_wait_response(
        "GetMainframeInformation", None
    )

    if (
        not any(
            key in response_json
            for key in [
                "MainframeType",
                "MainframeName",
                "SerialNumber",
                "FirmwareVersion",
            ]
        )
    ) or (response_json[RETURN_KEY] != GHSReturnValue["OK"]):
        return (
            to_string(response_json[RETURN_KEY], GHSReturnValue),
            None,
            None,
            None,
            None,
        )
    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["MainframeType"],
        response_json["MainframeName"],
        response_json["SerialNumber"],
        response_json["FirmwareVersion"],
    )


def get_disk_space(
    con_handle: ConnectionHandler,
) -> tuple[str, float | None, float | None]:
    """Interface to get total and available mainframe internal disk
    space.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       Tuple with status, total and available mainframe internal
       disk.
    """

    response_json = con_handle.send_request_wait_response("DiskSpace", None)

    if (
        not any(
            key in response_json
            for key in [
                "TotalSize",
                "AvailableSize",
            ]
        )
    ) or (response_json[RETURN_KEY] != GHSReturnValue["OK"]):
        return (
            to_string(response_json[RETURN_KEY], GHSReturnValue),
            None,
            None,
        )
    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["TotalSize"],
        response_json["AvailableSize"],
    )


def get_slot_count(
    con_handle: ConnectionHandler,
) -> tuple[str, int | None]:
    """Interface to get the number of slots in the mainframe.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       Tuple with status and number of slots in the mainframe.
    """

    response_json = con_handle.send_request_wait_response("GetSlotCount", None)

    if ("SlotCount" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None
    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["SlotCount"],
    )


def get_sync_status(
    con_handle: ConnectionHandler,
) -> tuple[str, str | None]:
    """Interface to determine the mainframe sync status.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       Tuple with status and sync status
    """

    response_json = con_handle.send_request_wait_response(
        "GetSyncStatus", None
    )

    if ("SyncStatus" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None
    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        to_string(response_json["SyncStatus"], GHSSyncStatus),
    )


def get_user_mode(
    con_handle: ConnectionHandler,
) -> tuple[str, str | None]:
    """Interface to retrieve the user mode.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       Tuple with status and user mode.
    """

    response_json = con_handle.send_request_wait_response("GetUserMode", None)

    if ("UserMode" not in response_json) or (
        response_json[RETURN_KEY] != GHSReturnValue["OK"]
    ):
        return to_string(response_json[RETURN_KEY], GHSReturnValue), None
    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        to_string(response_json["UserMode"], GHSUserMode),
    )


def set_user_mode(con_handle: ConnectionHandler, user_mode: str | int) -> str:
    """Interface to set the user mode.

    The system needs to be idle before calling this function.

    Args:
        con_handle: A unique identifier per mainframe connection.
        user_mode: The desired user mode.

    Returns:
       String value representing request status.
    """

    if isinstance(user_mode, str) and user_mode in GHSUserMode:
        user_mode_dict = {"UserMode": from_string(user_mode, GHSUserMode)}

    elif isinstance(user_mode, int) and user_mode in GHSUserMode.values():
        user_mode_dict = {"UserMode": user_mode}

    else:
        return "InvalidUserMode"

    response_json = con_handle.send_request_wait_response(
        "SetUserMode", user_mode_dict
    )

    return to_string(response_json[RETURN_KEY], GHSReturnValue)


def identity(con_handle: ConnectionHandler, identity_flag: bool) -> str:
    """Interface to enable or disable the identification sound of the
    connected mainframe.

    Args:
        con_handle: A unique identifier per mainframe connection.
        identity_flag: Enable or disable flag.

    Returns:
       String value representing request status.
    """

    identity_dict = {"Identify": 1} if identity_flag else {"Identify": 0}

    response_json = con_handle.send_request_wait_response(
        "Identify", identity_dict
    )

    return to_string(response_json[RETURN_KEY], GHSReturnValue)



# Copyright (C) 2022 Hottinger Bruel and Kjaer Benelux B.V.
# Schutweg 15a
# 5145 NP Waalwijk
# The Netherlands
# http://www.hbm.com

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Manage mainframe settings module interface."""

from .connection import ConnectionHandler
from .ghsapi_states import RETURN_KEY, GHSReturnValue, to_string


def apply_persisted_settings(con_handle: ConnectionHandler) -> str:
    """A mainframe might contain persisted settings (being applied upon
    boot). This method re-applies these settings. In Perception this
    maps on the 'Configured boot' feature.

    The system needs to be idle before calling this function.
    This function overwrites any previously set settings and / or
    persisted settings.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       String value representing request status.
    """

    response_json = con_handle.send_request_wait_response(
        "ApplyPersistedSettings", None
    )
    return to_string(response_json[RETURN_KEY], GHSReturnValue)


def persist_current_settings(con_handle: ConnectionHandler) -> str:
    """Persists the current mainframe settings.

    The persisted mainframe settings are applied upon a mainframe boot.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
       String value representing request status.
    """

    response_json = con_handle.send_request_wait_response(
        "PersistCurrentSettings", None
    )
    return to_string(response_json[RETURN_KEY], GHSReturnValue)


def get_current_settings(
    con_handle: ConnectionHandler,
) -> tuple[str, bytes | None, int | None]:
    """Retrieves the current mainframe settings as a blob.

    As this blob can be of variable size, it is upon the caller to
    ensure reading the correct amount of memory.
    Memory for this blob is allocated by the API.

    Args:
        con_handle: A unique identifier per mainframe connection.

    Returns:
        Tuple with status, base name and index of the recording file.
    """

    response_json = con_handle.send_request_wait_response(
        "GetCurrentSettings", None
    )

    if (
        not any(
            key in response_json
            for key in [
                "Size",
                "Blob",
            ]
        )
    ) or (response_json[RETURN_KEY] != GHSReturnValue["OK"]):
        return (
            to_string(response_json[RETURN_KEY], GHSReturnValue),
            None,
            None,
        )
    return (
        to_string(response_json[RETURN_KEY], GHSReturnValue),
        response_json["Blob"],
        response_json["Size"],
    )


def set_current_settings(
    con_handle: ConnectionHandler, blob: bytes, blob_size: int
) -> str:
    """Applies the mainframe settings contained in the input argument.

    Note that this function overwrites any previously set settings and
    / or applied setup.

    The system needs to be idle before calling this function.

    Args:
        con_handle: A unique identifier per mainframe connection.
        blob: Settings blob.
        blob_size: Size of the settings blob.

    Returns:
        String value representing request status.
    """

    if not blob or not blob_size:
        return "NullPtrArgument"

    current_settings_dict = {"Blob": blob, "Size": blob_size}
    response_json = con_handle.send_request_wait_response(
        "SetCurrentSettings", current_settings_dict
    )

    return to_string(response_json[RETURN_KEY], GHSReturnValue)
